import Flutter
import UIKit
import Berbix
import CommonCrypto

public class SwiftBerbixFlutterPlugin: NSObject, FlutterPlugin {
    private let berbix = BerbixSDK()
    private var config: BerbixConfiguration?
    private var currentFlowResult: FlutterResult?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        // This channel name is shared between dart, android and iOS. Don't change this unless it's
        // changed in the remaining platform files.
        let channel = FlutterMethodChannel(name: "flutter.berbix.com/sdk", binaryMessenger: registrar.messenger())
        let instance = SwiftBerbixFlutterPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        print("Received flutter method call \(call.method)")
        
        if call.method == "getPlatformVersion" {
            result("iOS " + UIDevice.current.systemVersion)
        } else if call.method == "setClientToken" {
            guard let args = call.arguments as? [String: String], let clientToken = args["clientToken"] else {
                print("Called setClientToken with an missing token!")
                return result(false)
            }
            setClientToken(clientToken, result)
        } else if call.method == "startFlow" {
            startFlow(result)
        }
    }
    
    private func setClientToken(_ clientToken: String, _ result: FlutterResult) {
        print("Updating iOS client identifier to \(clientToken)")
        config = BerbixConfigurationBuilder()
            .withClientToken(clientToken)
            .build()
        result(true)
    }
    
    private func startFlow(_ result: FlutterResult) {
        if let rootVC = UIApplication.shared.keyWindow?.rootViewController, let config = config {
            print("Starting flow with root view controller: \(rootVC)")
            currentFlowResult = result
            berbix.startFlow(rootVC,
                             delegate: self,
                             config: config)
            result(true)
        } else {
            print("Failed to start flow!")
            result(false)
        }
    }
}

extension SwiftBerbixFlutterPlugin: BerbixFlowDelegate {
    public func completed() {
        print("Finished flow")
        currentFlowResult(true)
    }
    
    public func failed(error: BerbixError) {
        print("Failed flow. Details: \(error.localizedDescription)")
        currentFlowResult(error)
    }
}